# -*- coding: UTF-8 -*-
__author__ = "Jaime Rivera <jaime.rvq@gmail.com>"
__copyright__ = "Copyright 2022, Jaime Rivera"
__credits__ = []
__license__ = "MIT License"


from PySide2 import QtCore, QtWidgets

from all_nodes import constants
from all_nodes import utils
from all_nodes.graphic.widgets.global_signaler import GlobalSignaler


GS = GlobalSignaler()

LOGGER = utils.get_logger(__name__)


BASIC_DATATYPES = sorted(
    ["bool", "dict", "float", "int", "list", "set", "str", "tuple"]
)
BASIC_DATATYPES_MAP = {
    "bool": bool,
    "dict": dict,
    "float": float,
    "int": int,
    "list": list,
    "set": set,
    "str": str,
    "tuple": tuple,
}

INPUTS_GUI = sorted([elem.value for elem in constants.InputsGUI])
INPUTS_GUI_MAP = {
    constants.InputsGUI.STR_INPUT.value: str,
    constants.InputsGUI.INT_INPUT.value: int,
    constants.InputsGUI.BOOL_INPUT.value: bool,
    constants.InputsGUI.FLOAT_INPUT.value: float,
}

OUTPUTS_GUI = sorted([elem.value for elem in constants.PreviewsGUI])
OUTPUTS_GUI_MAP = {
    constants.PreviewsGUI.STR_PREVIEW.value: str,
}


class AttributePicker(QtWidgets.QDialog):
    """TODO this description"""

    def __init__(self, *args, **kwargs):
        QtWidgets.QDialog.__init__(self, *args, **kwargs)

        self.setWindowTitle("Attribute Picker")

        self.layout = QtWidgets.QVBoxLayout()
        self.setLayout(self.layout)

        self.name_bar = QtWidgets.QLineEdit()
        self.name_bar.setPlaceholderText("Attribute name")
        self.layout.addWidget(self.name_bar)

        self.attribute_types_layout = QtWidgets.QHBoxLayout()

        basic_attrs_layout = QtWidgets.QVBoxLayout()
        basic_attrs_layout.addWidget(QtWidgets.QLabel("Basic types:"))
        self.basic_attrs_list = QtWidgets.QListWidget()
        self.basic_attrs_list.addItems(BASIC_DATATYPES)
        basic_attrs_layout.addWidget(self.basic_attrs_list)
        in_out_layout = QtWidgets.QHBoxLayout()
        self.in_checkbox = QtWidgets.QCheckBox(constants.INPUT)
        self.in_checkbox.setChecked(True)
        in_out_layout.addWidget(self.in_checkbox)
        self.out_checkbox = QtWidgets.QCheckBox(constants.OUTPUT)
        in_out_layout.addWidget(self.out_checkbox)
        basic_attrs_layout.addLayout(in_out_layout)
        self.attribute_types_layout.addLayout(basic_attrs_layout)

        inputs_layout = QtWidgets.QVBoxLayout()
        inputs_layout.addWidget(QtWidgets.QLabel("GUI Inputs:"))
        self.inputs_list = QtWidgets.QListWidget()
        self.inputs_list.addItems(INPUTS_GUI)
        inputs_layout.addWidget(self.inputs_list)
        self.attribute_types_layout.addLayout(inputs_layout)

        previews_layout = QtWidgets.QVBoxLayout()
        previews_layout.addWidget(QtWidgets.QLabel("GUI Previews:"))
        self.previews_list = QtWidgets.QListWidget()
        self.previews_list.addItems(OUTPUTS_GUI)
        previews_layout.addWidget(self.previews_list)
        self.attribute_types_layout.addLayout(previews_layout)

        self.layout.addLayout(self.attribute_types_layout)

        self.ok_button = QtWidgets.QPushButton("DONE")
        self.ok_button.setDisabled(True)
        self.layout.addWidget(self.ok_button)

        # Style
        f = QtCore.QFile(r"ui:stylesheet.qss")  # TODO not ideal, maybe a reduced qss?
        with open(f.fileName(), "r") as s:
            self.setStyleSheet(s.read())

        # Connections
        self.make_connections()

        self.exec_()

    def make_connections(self):
        self.name_bar.textChanged.connect(self.validate_inputs)

        self.basic_attrs_list.itemClicked.connect(self.validate_inputs)
        self.basic_attrs_list.itemClicked.connect(self.inputs_list.clearSelection)
        self.basic_attrs_list.itemClicked.connect(self.previews_list.clearSelection)

        self.inputs_list.itemClicked.connect(self.validate_inputs)
        self.inputs_list.itemClicked.connect(self.basic_attrs_list.clearSelection)
        self.inputs_list.itemClicked.connect(self.previews_list.clearSelection)

        self.previews_list.itemClicked.connect(self.validate_inputs)
        self.previews_list.itemClicked.connect(self.basic_attrs_list.clearSelection)
        self.previews_list.itemClicked.connect(self.inputs_list.clearSelection)

        self.in_checkbox.clicked.connect(self.out_checkbox.toggle)
        self.out_checkbox.clicked.connect(self.in_checkbox.toggle)

        self.ok_button.clicked.connect(self.close)

    def validate_inputs(self):
        if not self.name_bar.text():
            self.ok_button.setEnabled(False)
            return
        else:
            self.ok_button.setEnabled(True)

        if (
            not self.basic_attrs_list.selectedItems()
            and not self.inputs_list.selectedItems()
            and not self.previews_list.selectedItems()
        ):
            self.ok_button.setEnabled(False)

    def get_results(self):
        name = self.name_bar.text()

        if self.basic_attrs_list.selectedItems():
            attr_type = self.basic_attrs_list.currentItem().text()
            attr_connector = (
                constants.INPUT if self.in_checkbox.isChecked() else constants.OUTPUT
            )

            return [name, attr_connector, BASIC_DATATYPES_MAP.get(attr_type)]

        elif self.inputs_list.selectedItems():
            if not name.startswith("internal_"):
                name = "internal_" + name
            gui_type = self.inputs_list.currentItem().text()
            return [
                name,
                constants.INTERNAL,
                INPUTS_GUI_MAP.get(gui_type),
                constants.InputsGUI(gui_type),
            ]

        elif self.previews_list.selectedItems():
            if not name.startswith("internal_"):
                name = "internal_" + name
            gui_type = self.previews_list.currentItem().text()
            return [
                name,
                constants.INTERNAL,
                OUTPUTS_GUI_MAP.get(gui_type),
                constants.PreviewsGUI(gui_type),
            ]



__author__ = "Jaime Rivera <jaime.rvq@gmail.com>"
__copyright__ = "Copyright 2022, Jaime Rivera"
__credits__ = []
__license__ = "MIT License"


import os
import re

import matplotlib.pyplot as plt
import polars as pl
from pymongo import MongoClient

from all_nodes import constants
from all_nodes import utils


ENVIRONMENT = constants.DB_ENV
ALL_NODES_DB = "all_nodes"
ALL_NODES_TABLE = f"node_usage_{ENVIRONMENT}"

DB_READ_AND_WRITE_PASSWORD = os.getenv("DB_READ_AND_WRITE_PASSWORD")
DB_USERNAME = "read_and_write_user" if DB_READ_AND_WRITE_PASSWORD else "readonly_user"
DB_PASSWORD = DB_READ_AND_WRITE_PASSWORD or "jwNXfEqOBCiAPpbh"
CONNECTION_STRING = (
    f"mongodb+srv://{DB_USERNAME}:{DB_PASSWORD}@cluster0.hdqsbww.mongodb.net"
)

LOGGER = utils.get_logger(__name__)


# -------------------------------- METHODS -------------------------------- #
def get_collection():
    mongo_client = None
    try:
        mongo_client = MongoClient(
            CONNECTION_STRING,
            serverSelectionTimeoutMS=3000,
            connectTimeoutMS=3000,
            socketTimeoutMS=3000,
        )
        LOGGER.debug(mongo_client.admin.command("ping"))
        LOGGER.info(f"Connected to MongoDB for analytics, username: {DB_USERNAME}")
    except Exception as e:
        LOGGER.info(
            f"Cannot connect to MongoDB, no statistics will be submitted! ({e})"
        )
        return

    return mongo_client[ALL_NODES_DB][ALL_NODES_TABLE]


def make_query(query_dict: dict):
    """Make a query to the DB

    Args:
        query_str (dict): query to be made

    Returns:
        pymongo.cursor.Cursor
    """
    table = get_collection()
    if table is not None:
        return table.find(query_dict)

    return []


def make_query_aggregation(pipeline: list):
    """Make a query to the DB

    Args:
        pipeline (list): pipeline for query

    Returns:
        pymongo.cursor.Cursor
    """
    table = get_collection()
    if table is not None:
        return table.aggregate(pipeline)

    return []


def submit_bulk_analytics(node_attrs_list):
    """Submit a series of records to DB

    Args:
        node_attrs_list (list): list of dicts to be submitted to DB
    """
    if not DB_READ_AND_WRITE_PASSWORD:
        LOGGER.info("Cannot submit the stats of this run to the DB")
        return

    if not node_attrs_list:
        LOGGER.info("Nothing to submit")
        return

    table = get_collection()
    if table is not None:
        LOGGER.info(f"Submitting stats to {ALL_NODES_DB}.{ALL_NODES_TABLE}...")
        table.insert_many(node_attrs_list)


def process_analytics():
    """
    Create graphs with some node usage analytics
    """
    LOGGER.info(f"Getting statistics from {ALL_NODES_DB}.{ALL_NODES_TABLE}")

    # Folder
    root = os.path.abspath(__file__)
    root_dir_path = os.path.dirname(root)
    os.makedirs(os.path.join(root_dir_path, "../../../docs/analytics"), exist_ok=True)

    # Style for plots
    plt.style.use("seaborn-v0_8-dark")

    # Overall usage ---------------------------------------
    res = make_query_aggregation(
        [
            {"$match": {"run_date": {"$ne": None}}},
            {"$group": {"_id": "$run_date", "count": {"$sum": 1}}},
            {"$sort": {"_id": 1}},
            {"$limit": 35},
        ]
    )

    if res:
        df = pl.DataFrame(list(res))

        dates = df["_id"].to_list()
        uses = df["count"].to_list()

        fig, ax = plt.subplots(figsize=(10, 7))
        ax.plot(dates, uses)
        for tick in ax.get_xticklabels():
            tick.set_rotation(45)
            tick.set_horizontalalignment("right")
            tick.set_fontsize(9)
        ax.set_ylabel("Amount of nodes run")
        ax.set_title("Recent usage")

        graph_file = os.path.join(
            root_dir_path, "../../../docs/analytics", "recent_usage.png"
        )
        fig.tight_layout()
        fig.savefig(graph_file)

    # Most used ---------------------------------------
    res = make_query_aggregation(
        [
            {
                "$match": {
                    "$nor": [
                        {"class_name": re.compile(".*Input")},
                        {"class_name": re.compile(".*Ctx")},
                    ],
                    "IS_CONTEXT": {"$ne": True},
                }
            },
            {"$group": {"_id": "$class_name", "count": {"$sum": 1}}},
            {"$sort": {"count": -1}},
            {"$limit": 30},
        ]
    )

    if res:
        df = pl.DataFrame(list(res))

        node_names = df["_id"].to_list()
        uses = df["count"].to_list()

        fig, ax = plt.subplots(figsize=(10, 7))
        ax.barh(node_names, uses)
        ax.set_xlabel("Number of usages")
        ax.set_ylabel("Node class")
        ax.invert_yaxis()
        ax.set_title("Top 30 most used nodes")

        graph_file = os.path.join(
            root_dir_path, "../../../docs/analytics", "most_used.png"
        )
        fig.tight_layout()
        fig.savefig(graph_file)

    # Last errored ---------------------------------------
    res = make_query_aggregation(
        [
            {
                "$match": {
                    "success": {"$eq": "ERROR"},
                    "run_date": {"$ne": None},
                }
            },
            {"$sort": {"run_date": 1}},
            {"$limit": 50},
        ]
    )

    if res:
        df = pl.DataFrame(list(res))

        df_grouped = df.group_by(
            ["class_name", "run_date"], maintain_order=True
        ).count()

        node_names = df_grouped["class_name"].to_list()
        failed_date = df_grouped["run_date"].to_list()
        occurrences = df_grouped["count"].to_list()

        fig, ax = plt.subplots(figsize=(10, 7))
        ax.scatter(
            failed_date,
            node_names,
            c="red",
            alpha=0.5,
            s=[min(5000, 30 * o) for o in occurrences],
            edgecolors="black",
        )
        ax.set_xlabel("Error date")
        for tick in ax.get_xticklabels():
            tick.set_rotation(45)
            tick.set_horizontalalignment("right")
        ax.set_ylabel("Node class")
        ax.set_title("Most recently errored nodes")
        ax.grid(which="major", axis="x", linestyle="--")

        graph_file = os.path.join(
            root_dir_path, "../../../docs/analytics", "errored.png"
        )
        fig.tight_layout()
        fig.savefig(graph_file)

    # Last failed ---------------------------------------
    res = make_query_aggregation(
        [
            {
                "$match": {
                    "success": {"$eq": "FAILED"},
                    "run_date": {"$ne": None},
                }
            },
            {"$sort": {"run_date": 1}},
            {"$limit": 50},
        ]
    )

    if res:
        df = pl.DataFrame(list(res))

        df_grouped = df.group_by(
            ["class_name", "run_date"], maintain_order=True
        ).count()

        node_names = df_grouped["class_name"].to_list()
        failed_date = df_grouped["run_date"].to_list()
        occurrences = df_grouped["count"].to_list()

        fig, ax = plt.subplots(figsize=(10, 7))
        ax.scatter(
            failed_date,
            node_names,
            c="orange",
            alpha=0.5,
            s=[min(5000, 30 * o) for o in occurrences],
            edgecolors="black",
        )
        ax.set_xlabel("Failure date")
        for tick in ax.get_xticklabels():
            tick.set_rotation(45)
            tick.set_horizontalalignment("right")
        ax.set_ylabel("Node class")
        ax.set_title("Most recently failed nodes")
        ax.grid(which="major", axis="x", linestyle="--")

        graph_file = os.path.join(
            root_dir_path, "../../../docs/analytics", "failed.png"
        )
        fig.tight_layout()
        fig.savefig(graph_file)

    LOGGER.info(f"Processed statistics from {ALL_NODES_DB}.{ALL_NODES_TABLE}")



# -*- coding: UTF-8 -*-
__author__ = "Jaime Rivera <jaime.rvq@gmail.com>"
__copyright__ = "Copyright 2022, Jaime Rivera"
__credits__ = []
__license__ = "MIT License"


import argparse
import os
import sys

from PySide2.QtWidgets import QApplication

from all_nodes.analytics import analytics
from all_nodes.graphic.widgets.main_window import AllNodesWindow
from all_nodes.logic.class_registry import CLASS_REGISTRY as CR
from all_nodes.logic.logic_scene import LogicScene
from all_nodes import utils


LOGGER = utils.get_logger(__name__)


# LAUNCH MODES ---------------------------------------------------
def launch_gui():
    """
    Just launch the tool, with GUI to create/edit scenes.
    """
    # Start classes scannig first thing
    CR.scan_for_classes_GUI()

    # App
    app = QApplication(sys.argv)

    # Launch
    AllNodesWindow()
    app.exec_()


def launch_batch(scene_file: str, set_parameters: list):
    """
    Run a scene in batch mode, no GUI.

    Args:
        scene_file (str): Filepath or alias of the scene to run
        scene_file (list): List eith parameters and values to be set
    """
    # Start classes scannig first thing
    CR.scan_for_classes()

    # Scene
    scene = LogicScene()
    scene.load_from_file(scene_file)
    if set_parameters:
        for i in range(0, len(set_parameters) - 1, 2):
            node_name = set_parameters[i].rsplit(".", 1)[0]
            attr_name = set_parameters[i].rsplit(".", 1)[1]
            attr_str_value = set_parameters[i + 1]

            node = scene.to_node(node_name)
            if node:
                node.set_attribute_from_str(attr_name, attr_str_value)

    # Run!
    scene.run_all_nodes(spawn_thread=False)


# MAIN ---------------------------------------------------
def main():
    # Startup logging
    LOGGER.info("STARTED all_nodes")

    # Arguments ----------------------
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-f", "--scene_file", type=str, help="file .yml to run in non-GUI mode"
    )
    parser.add_argument(
        "-s",
        "--set_parameters",
        help="Arguments to set in batch execution of scene",
        type=str,
        nargs="+",
    )
    parser.add_argument(
        "-a",
        "--analytics",
        help="Perform analytics and generate graphs",
        action="store_true",
    )
    args = parser.parse_args()

    if not os.getenv("IN_DEV"):
        LOGGER.info("For launching in DEBUG mode, set env variable 'IN_DEV'")

    # Analytics ----------------------
    if args.analytics:
        analytics.process_analytics()
        sys.exit(0)

    # GUI mode ----------------------
    if not args.scene_file:
        launch_gui()

    # Non-GUI batch mode ----------------------
    else:
        launch_batch(args.scene_file, args.set_parameters)


if __name__ == "__main__":
    main()



# -*- coding: UTF-8 -*-
__author__ = "Jaime Rivera <jaime.rvq@gmail.com>"
__copyright__ = "Copyright 2022, Jaime Rivera"
__credits__ = []
__license__ = "MIT License"


import logging
import re
import sys

from colorama import Fore, Style
from PySide2 import QtGui

from all_nodes import constants


# -------------------------------- LOGGING -------------------------------- #
def get_logger(logger_name: str) -> logging.Logger:
    """Get a logger with a specific formatting

    Args:
        logger_name (str)

    Returns:
        logging.Logger
    """
    logger = logging.getLogger(logger_name)
    logger.setLevel(constants.LOGGING_LEVEL)
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setLevel(constants.LOGGING_LEVEL)
    console_handler.setFormatter(constants.CONSOLE_LOG_FORMATTER)
    logger.addHandler(console_handler)
    return logger


LOGGER = get_logger(__name__)


def print_separator(message):
    """Print a separator to screen

    Args:
        message (str): Message to display in the separator
    """
    LOGGER.info(f"\n{message.upper()}\n{'-' * len(message)}")


def print_test_header(message):
    """Print a test header separator to screen

    Args:
        message (str): Message to display in the separator
    """
    header = f"TEST STARTED - {message}"
    print(
        f"{Fore.GREEN}+-{'-' * len(header)}-+\n"
        f"{Fore.GREEN}| {header} |\n"
        f"{Fore.GREEN}+-{'-' * len(header)}-+ {Style.RESET_ALL}"
    )


# -------------------------------- LOGIC UTILITY -------------------------------- #
def parse_datatype(datatype_str):
    if re.match("<class '.+\.(.+)'>", datatype_str):
        return re.match("<class '.+\.(.+)'>", datatype_str).group(1)
    elif re.match("<module '.+\.(.+)' from", datatype_str):
        return re.match("<module '.+\.(.+)' from", datatype_str).group(1)
    return re.search("'(.+)'", datatype_str).group(1)


# -------------------------------- GUI UTILITY -------------------------------- #
def get_bright_color(color_name):
    """Given a color name, get a fully saturated and bright version of it

    Args:
        color_name (str): name of the color to saurate

    Returns:
        str: name of the saturated color
    """
    base_color = QtGui.QColor(color_name)
    h = base_color.hue()

    bright_color = QtGui.QColor.fromHsv(h, 255, 255)
    return bright_color.name()



__author__ = "Jaime Rivera <jaime.rvq@gmail.com>"
__copyright__ = "Copyright 2022, Jaime Rivera"
__credits__ = []
__license__ = "MIT License"


from PySide2 import QtCore


class GlobalSignals(QtCore.QObject):
    # -------------------------- LOGIC -------------------------- #
    class_scanning_finished = QtCore.Signal()

    # -------------------------- WIDGETS -------------------------- #
    # Widget move ----------------------
    class_searcher_move = QtCore.Signal(int, int)

    # Tabs ----------------------
    tab_names_refresh_requested = QtCore.Signal()

    # Attribute editor ----------------------
    attribute_editor_node_addition_requested = QtCore.Signal(str)
    attribute_editor_refresh_node_requested = QtCore.Signal(str)
    attribute_editor_remove_node_requested = QtCore.Signal(str)

    attribute_editor_global_refresh_requested = QtCore.Signal()

    # Contexts ----------------------
    context_expansion_requested = QtCore.Signal(str)

    # Node creation ----------------------
    dropped_node = QtCore.Signal(QtCore.QPoint)
    node_creation_requested = QtCore.Signal(QtCore.QPoint, str)

    # Feedback ----------------------
    main_screen_feedback = QtCore.Signal(str, int)

    # -------------------------- EXECUTION -------------------------- #
    execution_started = QtCore.Signal()
    execution_finished = QtCore.Signal()


class GlobalSignaler:
    # TODO should this signaler be somewhere else? It is not a widget
    __instance = None

    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = super(GlobalSignaler, cls).__new__(cls)
            cls.signals = GlobalSignals()
        return cls.__instance



__author__ = "Jaime Rivera <jaime.rvq@gmail.com>"
__copyright__ = "Copyright 2022, Jaime Rivera"
__credits__ = []
__license__ = "MIT License"


from PySide2 import QtWidgets
from PySide2 import QtCore

from all_nodes import utils
from all_nodes.graphic.widgets.global_signaler import GlobalSignaler


GS = GlobalSignaler()

LOGGER = utils.get_logger(__name__)


SHORTCUTS_DICT = {
    "Examine selected nodes code": "E",
    "Run selected nodes": "Return",
    "Reset selected nodes": "R",
    "Toggle selected nodes activation": "D",
    "Expand selected contexts": "Ctrl + Return",
    "Export selected nodes code": "Ctrl + E",
    "Soft-reset selected nodes": "S",
    "Delete selected nodes": "Del",
    "Fit to view": "F",
    "Search node class": "Tab",
}


class ShortcutsHelp(QtWidgets.QWidget):
    """Small helper widget to show the keyboard shortcuts."""

    def __init__(self, *args, **kwargs):
        QtWidgets.QWidget.__init__(self, *args, **kwargs)

        self.layout = QtWidgets.QVBoxLayout()
        self.setLayout(self.layout)

        self.layout.addWidget(QtWidgets.QLabel("KEYBOARD SHORTCUTS"))

        for action, key_shortcut in SHORTCUTS_DICT.items():
            h_layout = QtWidgets.QHBoxLayout()
            h_layout.addWidget(QtWidgets.QLabel(" " + action))
            shortcut_label = QtWidgets.QLabel(key_shortcut)
            shortcut_label.setFixedWidth(120)
            shortcut_label.setAlignment(QtCore.Qt.AlignCenter)
            shortcut_label.setStyleSheet(
                "background-color:rgba(0,0,0,150); border-radius:5px;"
            )
            h_layout.addWidget(shortcut_label)
            self.layout.addLayout(h_layout)



__author__ = "Jaime Rivera <jaime.rvq@gmail.com>"
__copyright__ = "Copyright 2022, Jaime Rivera"
__credits__ = []
__license__ = "MIT License"


import time

from all_nodes.constants import PreviewsGUI
from all_nodes.logic.logic_node import GeneralLogicNode
from all_nodes import utils


LOGGER = utils.get_logger(__name__)


class EmptyNode(GeneralLogicNode):
    NICE_NAME = "Empty node"
    HELP = "Does not do anything (just debugging purposes)"

    def run(self):
        LOGGER.info("All good! Nothing to be done :)")


class ErrorNode(GeneralLogicNode):
    def run(self):
        100 / 0


class FailNode(GeneralLogicNode):
    def run(self):
        self.fail("This node is supposed to fail")
        self.fail("Also, more than one fail message can be logged")


class FailAndErrorNode(GeneralLogicNode):
    def run(self):
        self.fail("This node has failed")
        [10][5]


class TimedNode(GeneralLogicNode):
    NICE_NAME = "Timed node"
    HELP = "Node that sleeps for an amount of time"

    INPUTS_DICT = {
        "sleep_time": {"type": float, "optional": True},
    }

    def run(self):
        time.sleep(self.get_attribute_value("sleep_time") or 1.5)


class PrintToConsole(GeneralLogicNode):
    NICE_NAME = "Print to console"
    HELP = "Print something to console"

    INPUTS_DICT = {
        "in_object_0": {"type": object, "optional": True},
        "in_object_1": {"type": object, "optional": True},
        "in_object_2": {"type": object, "optional": True},
        "in_object_3": {"type": object, "optional": True},
        "in_object_4": {"type": object, "optional": True},
    }

    def run(self):
        from colorama import Fore, Style

        for i in range(5):
            attr_name = "in_object_{}".format(i)
            val = self.get_attribute_value(attr_name)
            if val is not None:
                LOGGER.info(
                    f"{Fore.MAGENTA}[{self.node_name}] {attr_name}:{Fore.YELLOW}{val}{Style.RESET_ALL}"
                )


class IntAddition(GeneralLogicNode):
    INPUTS_DICT = {
        "in_int_0": {"type": int},
        "in_int_1": {"type": int},
        "in_int_2": {"type": int, "optional": True},
        "in_int_3": {"type": int, "optional": True},
        "in_int_4": {"type": int, "optional": True},
    }

    OUTPUTS_DICT = {"out_total": {"type": int}}

    INTERNALS_DICT = {
        "internal_str": {"type": str, "gui_type": PreviewsGUI.STR_PREVIEW},
    }

    def run(self):
        int_0 = self.get_attribute_value("in_int_0")
        int_1 = self.get_attribute_value("in_int_1")
        int_2 = self.get_attribute_value("in_int_2")
        int_3 = self.get_attribute_value("in_int_3")
        int_4 = self.get_attribute_value("in_int_4")

        total = int_0 + int_1 + (int_2 or 0) + (int_3 or 0) + (int_4 or 0)

        self.set_output("out_total", total)
        self.set_attribute_value("internal_str", f"Total: {total}")
